;----------------------------------------------------------------------------;
; Kapazitts und ESR Messgert Hanno Grff Januar 2010
; nach 'Capacitance Meter Control Program R0.01' von Elm-Chan, 2003
; und ELV Elektronik AG ESR-Messgert ESR1, 2002
; 
;
;----------------------------------------------------------------------------;

.include "m48def.inc"
.include "avr.inc"


;****** Eeprom area *********************************************************
.eseg	
	
eDaten:
	 .db 0x7B,0x96,0xDE,0x91,0x80,0xFF

;****** Data memory area ****************************************************


.dseg
	.org SRAM_START

	DispPtr: .byte	1	;
	DispBuf: .byte	6	;
	KeyScan: .byte	2	;

	Comp1:	 .byte	2	;Range low compensation
	Comp2:	 .byte	2	;Range high compensation
	Comp3:	 .byte	2	;Zero compensation value

	StrBuf:	 .byte	10	;Decimal conversion buffer


;****** Program code area ***************************************************

.cseg
	.org 0
	rjmp	reset		;Reset
	reti				;External INT0
	reti				;External INT1
	reti				;Pin_Change 0
	reti				;Pin_Change 0
	reti				;Pin-Change 1
	reti				;Watchdog
	reti				;TC2 compare 
	reti				;TC2 compare
	reti				;TC2 overflow
	rjmp	tc1_cap		;TC1 Capture event
	reti				;TC1 compare 
	reti				;TC1 compare
	rjmp	tc1_ovf		;TC1 overflow
	reti				;TC0 compare 
	reti				;TC0 compare
	rjmp	tc0_ovf		;TC0 overflow


;****** Initialisierung *******************************************************

reset:
	ldi		AL, LOW(RAMEND)
	out		SPL,AL
	ldi		AL, high(RAMEND)
	out		SPH,AL
	clr		Zero					

	ldi		AL, 0b00111111			; Ausgnge LCD-Port
	out		DDRD, AL					
	
	ldi		AL, 0b01000000			; Pullup fr AIN0
	out		PortD,AL	

	ldi		AL, 0b00111000			; PullUps fr 3,4,5,6
	out		PortC, AL

	ldi		AL, 0b00101110			; PORTC: C0(FET) C2,C3,C5 auf Ausgang
	out		DDRC, AL

	ldi		AL, 0b00001101			; PortB: B0 und B2 = Kalibrierung, PB1= PWM-Oscillator , PB3 = Schalter Cap/ESR
	out		PortB,AL
	
	ldi		AL, 0b00000010			; OC1A = Ausgang
	out		DDRB,AL

	rcall 	lcd_init				; LCD initialisieren
	rcall	lcd_clear				; Display lschen
	rcall	Wait100ms				; warten, sonst leidet Eeprom 


;****** Vorbereitungen Kapazitt *********************************************

cap_main:
	
	
	ldi		AL,0x40					; 
	rcall	lcd_Locate				; ESR-Zeile ggf. Lschen
	ldi		ZL,Low(Leer*2)  		
  	ldi   	ZH,High(Leer*2)		
	rcall	lcd_print
	
	sts		TCCR1A, Zero	
	sts		TCCR1B, Zero

	ldi		AL, (1<<CS02)  

	out		TCCR0B, AL				; ClK/256 => TC0 = 39kHz

	ldi		AL, (1<<TOIE0)			; Timer/Counter0 Overflow Interrupt setzen
	sts		TIMSK0, AL				

	ldi		AL,(1<<ACIC)
	out		ACSR, AL				; Analog Comparator, AC Input Capture

	sei			

	rcall	load_eep				; Kalibrierungswerte laden
	breq	cm0						; wenn vorhanden, weiter zu main
	
    ldi		ZL,low(form3*2)			; sonst Eeprom-Fehler ausgeben
	ldi		ZH,high(form3*2)
	
	rcall	lcd_home				
	rcall	lcd_print				; 
	rcall 	wait1s
	rcall	lcd_clear				; LCD lschen


;****** Hauptprogramm Kapazitt *********************************************

cm0:ldi		AL, 25					; 100ms warten 
	mov		Stm1, AL		
	cbr		Flags, bit7				; ggf. Tastendruck	auswerten
	sbrc	Flags, 7		
	rjmp	btn_pressed				; O-Punkt oder Kalibrierung
	tst		Stm1			
	brne	PC-3			
	tst		Stm2					; bis Timer2 beendet
	brne	PC-5			
	brne	PC-4
		
	ldi		AL, 125					; Start Timer2 (500ms)
	mov		Stm2, AL		
	rcall	measure					; Messen im unteren Bereich
	brcc	cm1						; wenn kein Overflow weiter bei cm1

	cbi		PORTC,3					; PINB3 auf LOW setzen
	rcall	measure					; Messen im mittleren Bereich
	brcc	cm1						; wenn kein Overflow weiter bei cm1

	ldi		AL, 125					; sonst wieder bei unten anfangen
	mov		Stm2, AL		
	sbi		PORTC,3					; PINB3 auf HIGH, PINB5 auf HIGH
	sbi		PORTC,5					; 
	rcall	measure					; Messen im unteren Bereich
	brcc	cm1						; wenn kein Overflow weiter bei cm1
	
	sbi		PortC,3					; sonst PINB3 auf HIGH, PINB5 auf LOW
	cbi		PortC,5					; Messen im oberen Bereich
	rcall	measure					

cm1:
	sbis	PINC,4					; auf Tastendruck prfen
	rjmp	btn_pressed				; O-Punkt oder Kalibrierung
	
	sbis	PINB,3					; auf Schalter ESR prfen
	rjmp	esr_main
	
	rcall	adjust_zero				; sonst Messung auswerten
	rcall	adjust_gain				
	rcall	disp_val				; und anzeigen
	rcall	disp_measure
	
	sbi		PORTC,3					; PINB3 auf HIGH, PINB5 auf HIGH
	sbi		PORTC,5					; => Messen im unteren Bereich
	rjmp	cm0


;****** Vorbereitungen ESR ***************************************************

esr_main:
	
	cli
	
	; Timer/Counter Interrupts ausschalten
	sts		TIMSK0, Zero
	sts		TIMSK1, Zero
	out		ACSR, Zero

	; Fast PWM Mode 14
	ldi		AL, (1<<COM1A1)|(1<<COM1A0)|(1<<COM1B1)|(0<<COM1B0)|(1<<WGM11)|(0<<WGM10)
	sts		TCCR1A, AL
	
	ldi		AL, (0<<ICNC1)|(0<<ICES1)|(1<<WGM13)|(1<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10)
	sts		TCCR1B, AL

	; 80/160 = 60khz bei 10MHZ  60/120 = 80khz 98/194 = 50khz bei 10Mhz 49/98 = 100 khz
	sts		OCR1AH,Zero
	ldi		AL,60
	sts		OCR1AL,AL   	

	sts		ICR1H, Zero
	ldi		AL, 120
	sts		ICR1L, AL
	
		
	ldi		AL,0x40
	rcall	lcd_Locate				; LCD 2.Zeile 1. Stelle
	ldi		ZL,Low(Form1*2)  		; "ESR" ausgeben
  	ldi   	ZH,High(form1*2)		
	rcall	lcd_print
		
	sei


;****** Hauptprogramm ESR ***************************************************	

em0:	
	ldi		AL, 0xC0 				; 0xC0=Int
	rcall 	Get_Voltage
	ldi		AL,0xFF					; Maximum 03FF?
	ldi		AH,3
	cp		BL,AL					; wenn ja-> Overflow ausgeben
	cpc		BH,AH
	breq	em1
	mov		Al,BL
	mov		AH,BH
	rcall 	bin2ascii				
	rcall 	print_voltage
	rcall	wait100ms
	sbic	PINB,3					; Ist PINB3 gesetzt
	rjmp	Cap_Main				; wenn ja zurck zu Kapazittsmessung
	rjmp	em0						; sonst bei ESR bleiben
em1:
	
	ldi		AL,0x45				; 
	rcall	lcd_Locate				; LCD 2. Zeile 5. Stelle ggf. Lschen
	ldi		ZL,Low(overflow*2)
  	ldi   	ZH,High(Overflow*2)
	rcall	lcd_print
		
	sbic	PINB,3					; Ist PINB3 gesetzt?
	rjmp	cap_main				; wenn ja zurck zu Kapazittsmessung
	rjmp	em0						; sonst bei ESR bleiben



;****** Unterprogramme Kapazitt ********************************************
;
; Tastendruck, Kalibrierung, Offset

btn_pressed:						; Taste gedrckt ?
	ldi		AL, 4					; Delay 16ms
	rcall	dly				
	sbis	PINB, 0					; Ist PB0 berbrckt?
	rjmp	cal_low					; Ja=> Kalibrierung Low-Range
	sbis	PINB, 2					; Ist PB2 berbrckt?
	rjmp	cal_high		        ; JA=> Kalibrierung Mid-Range
	rjmp	can_offset				; sonst Nullpunkt setzen

cal_high:
	cbi		PORTC, 3				; Mid-Range setzen
	rcall	measure					; messen
	sbi		PORTC, 3		
	ldiw	C, 0					; X:D:C = 1000*65536; (100nF reference cap)
	ldiw	D, 1000			
	ldiw	X, 0			
	ldiw	Z, Comp2				; Speicherstelle holen
	rjmp	cal_comp

cal_low:
	rcall	measure					; messen
	rcall	adjust_zero				
	ldiw	C, 0					; X:D:C = 10000*65536; (1nF reference cap)
	ldiw	D, 10000		
	ldiw	X, 0			
	ldiw	Z, Comp1

cal_comp:
	clrw	T0						; X:D:C /= B:A;
	clrw	T2				
	ldi		EL, 48			
	lslw	C				
	rolw	D				
	rolw	X				
	rolw	T0				
	rolw	T2				
	cpw		T0, A			
	cpcw	T2, B			
	brcs	PC+6			
	subw	T0, A			
	sbcw	T2, B			
	inc		CL				
	dec		EL					
	brne	PC-21			
	or		DL, DH					; Check over flow
	or		DL, XL				
	or		DL, XH				
	brne	cal_err					
	stdw	Z+0, C
  	rcall	clr_disp
	rcall	save_eep
	rjmp	cm0

can_offset:
	rcall	measure					; Measure capacitance as zero
	or		BL, BH					; Check adjustment range
	brne	cal_err					;
	cpi		BH, high(2000)			;
	brcc	cal_err					; 
	stsw	Comp3, A				; Set the value as zero point
	rcall	clr_disp
	rjmp	cm0

cal_err:
	rcall	lcd_home
	ldiw	Z, form4*2				; Fehler ausgeben
	;cli
	rcall 	lcd_print
	;sei
	ldi		AL, 250
	rcall	dly
	rcall	lcd_clear
	rjmp	cm0

dly:
	mov		Stm1, AL
	tst		Stm1	
	brne	PC-1
	ret


;****** Messung Kapazitt ***************************************************

measure:
	;cli
	sbi		PORTC,1						; Kondensator entladen
	rcall	wait100ms
	cbi		PORTC,1
	;sei
	
	sts		TCNT1H, Zero				; Clear TC1 and set time limit
	sts		TCNT1L, Zero				;
	clr		T2L							;
	
	ldi		AL, 20						; Zeit 130 ms = 57 nF
	sbis	PORTC,3						; berspringe, wenn PortB3 gesetzt
	ldi		AL,152						; 1000ms = 440 uF
	sbis	PortC,5						; berspringe, wenn PortB5 gesetzt
	ldi		AL, 152						; 1000ms
	
	mov		T2H, AL						; T2H ist Zhler in TC Overflow interrupt
	ldi		AL, 0x21; (1<<TOV1)|(1<<ICF1)
	out		TIFR1, AL					
	ldi		AL, (1<<TOIE1)|(1<<ICIE1)
	sts		TIMSK1,AL					; T/C Overflow Interrupt 1, ICE1, Intrpt 0
	ldi		AL,(1<<TOIE0)
	sts		TIMSK0,AL

	cbr		Flags, bit0+bit1+bit2		;
	ldi		AL, (1<<ICES1)|(1<<CS10)
	sts		TCCR1B, AL					; ICES1 CS10 Timer Capture / no prescaling
	cbi		DDRD, 6						; Start to charge (AIN0)
	
	sbrc	Flags, 2					; Flag=2 = overflow
	rjmp	mea_over					;
	sbrs	Flags, 1					; Flag=1 = negative Value
	rjmp	PC-3						; 

	ldi		AL, (1<<ICES1)
	sts		TCCR1B, AL					; Stop TC1
	movew	A, T4						; Ergebnis in AH:AL
	movew	B, T6						; und BH:BL
	clc
	ret

mea_over:
	ldi		AL, (1<<ICES1)
	sts		TCCR1B, AL					; Stop TC1
	ldi		Al,4						
	rcall	dly
	ldi		BH, -1						; Overflow
	sec                        			; set Carry-Flag
	ret

adjust_zero:
	sbis	PORTC, 3					; Skip if in mid range or high range
	rjmp	az0
	sbis	PortC, 5
	rjmp	az0							; 
	ldsw	C, Comp3					; B:A -= Comp3;
	subw	A, C						;
	sbc		BL, Zero					
	sbc		BH, Zero					
	brcc	az0							; if sign, B:A *= -1; and set sign flag.
	sbr		Flags, bit0					
	comw	A							
	comw	B							
	adc		AL, Zero					;
	adc		AH, Zero					;
	adc		BL, Zero					;
	adc		BH, Zero					;
az0:
	ret

adjust_gain:
	ldiw	Y, Comp1
	sbis	PORTC,3						; Gain adjustment
	adiw	YL,2
	sbis	PortC,5
	adiw	YL, 2						; Load compensation value in to D by range
	lddw	D, Y+0						; 
	subw	C, C						; B:A = B:A * D / 65536;
	ldi		EL, 33						;
ag0:
	brcc	ag1
	addw	C, D						;
ag1:
	rorw	C							;
	rorw	B							;
	rorw	A							;
	dec		EL							;
	brne	ag0
	movew	A, B						;
	movew	B, C						;
	ret


;****** Ergebnis-Ausgabe (B:A in pF) **************************************** 

disp_val:
	ldiw	X, StrBuf					; Puffer
	clr		DL							; Anzahl Stellen
dv1:	
	inc		DL							; Zhler1 = Zhler1 + 1
	clr		CL							; Zhler2 lschen
	ldi		CH,32						; Zhler2=32 	
dv0:
	lslw	A							; A
	rolw	B							; und B
	rol		CL							; und CL * 2
	cpi		CL,10						; wenn CL<10 weiter mit schieben
	brcs	dv3							; 	
	subi	CL,10						; sonst CL=CL-10
	inc		AL							; Rest unten nachschieben
dv3:
	dec		CH							; Zhler2=Zhler2-1
	brne	dv0							; zurck zum Anfang	
	st		X+, CL						; Wert in CL sichern
	cp		AL, Zero					; ist Al	
	cpc		AH, Zero					; und Ah	
	cpc		BL, Zero					; und Bl
	cpc		BH, Zero					; und BH =0
	brne	dv1							; wenn nicht zurck
	cpi		DL, 2						; ist Zhler2 = 2
	brcc	dv2							; wenn ja weiter
	st		X+, Zero					; wenn nicht
	inc		DL							; Zhler1 erhhen
dv2:	
	sbis	PORTC, 3					; 
	addi	DL, 3
	sbis	PortC, 5
	addi	DL, 4						; 
	ldiw	Z, form2*2-6		 
	sbrs	Flags, 0					; ist Bit0 von Flags = 0 dann Werte>0
	adiw	ZL, 6

dv4:
	adiw	ZL, 6						; so viele Masken
	dec		DL							; wie Stellen
	cpi		DL, 2						; addieren
	brcc	dv4

put_formed:
	clr		AH
	ldi		YL, Low(DispBuf)
	ldi		YH, High(DispBuf)
pf0:
	lpm
	adiw	ZL, 1
	mov		AL, T0L
	clt
	
	tst		AL
	brne	pf2
	bst		AL, 0
	ld		AL, -X
pf2:
	pushw	Z
	ldiw	Z, seg7*2
	addw	Z,A
	lpm
	popw	Z
	st		Y+, T0L
	cpi		YL, low(DispBuf+6)
	brne	pf0
	ret

clr_disp:
	ldi		YL,low(Dispbuf)
	ldi		Yh,High(DispBuf)
	st		Y+, Zero
	cpi		YL, low(DispBuf+6)
	brne	PC-2
	ret

disp_measure:
	;cli
	rcall	lcd_Home
	ldi		ZL,low(form0*2)
	ldi		ZH,high(form0*2)
	rcall	lcd_print
	ldi		ZL,Low(Dispbuf)
	ldi		ZH,High(Dispbuf)
	
d0:	ld		AL,Z+	
	rcall	lcd_Data					; Ausgabe
	cpi		ZL,low(DispBuf+6)			; ist Zhler=0 ?
	brne	d0                			; wenn nicht, nchstes Zeichen
	;sei
	ret



;****** Load/Save EEPROM ****************************************************

load_eep:
	ldi		YL,low(Comp1)				; Adresse 0x109	
	ldi		YH,high(Comp1)
	ldi		CL,Low(eDaten)				; Read-Eprom
	ldi		CH,High(eDaten)
le1:	
	rcall 	read_eep
	st		Y+, AL						; Daten aus Eeprom ins SRAM kopieren
	add		CH, AL
	cpi		YL, low(comp1+4)  			; 4 Bytes
	brne	le1
	rcall	read_eep					; Check SUM
	cp		AL, CH		
	breq	PC+6		
	ldi		AL,-1
	st		-Y, AL						; Set default value if data have been broken.
	st		-Y, AL		
	st		-Y, AL		
	st		-Y, AL		
	ret


save_eep:
	ldiw	Y, Comp1					; Speicher im SRAM von Comp1
	ldiw	C, edaten					; ins Eeprom schreiben
	ld		AL, Y+		
	add		CH, AL		
	rcall	write_eep	
	cpi		YL, low(Comp1+4)
	brne	PC-4		
	mov		AL, CH						; Save check SUM
	

write_eep:
	out		EEARL, CL
	inc		CL
	out		EEDR, AL
	cli
	sbi		EECR, EEMPE
	sbi		EECR, EEPE
	sei
	sbic	EECR, EEPE
	rjmp	PC-1
	ret

read_eep:
	out		EEARL, CL
	inc		CL
	sbi		EECR, EERE
	in		AL, EEDR
	ret


;****** Timer-Counter1 Overflow Interrupt ***********************************

tc1_ovf:
	push	AL
	in		AL, SREG
	push	AL
	inc		T2L
	cp		T2L, T2H
	brcs	PC+6
	sbi		DDRC, 2
	sbi		DDRD, 6
	sbr		Flags, bit2
	ldi		AL,(1<<TOIE0) 
	sts		TIMSK0, AL
	pop		AL
	out		SREG, AL
	pop		AL
	reti



;****** Timer-Counter1 Capture Interrupt ************************************

tc1_cap:
	push	AL
	in		AL, SREG
	push	AL
	sbis	DDRC, 2							; Branch by measureing phase.
	rjmp	tc1c_ed				 

tc1c_st:									; Vc reaches 0.17 Vcc
	lds		T4L, ICR1L						; Capture t1
	lds		T4H, ICR1H			
	mov		T6L, T2L			 
	cbi		DDRC, 2							; Change Vth to 0.5 Vcc.
	ldi		AL, 20							; Deley several microseconds and clear Irq.
	dec		AL					
	brne	PC-1				
	ldi		AL, (1<<ICF1)
	out		TIFR1, AL
	rjmp	tc1c_e

tc1c_ed:									; Vc reaches 0.5 Vcc
	mov		T6H, T4L						; Capture t2-t1
	lds		T4L, ICR1L			
	sub		T4L, T6H			
	mov		T6H, T4H			
	lds		T4H, ICR1H			
	sbc		T4H, T6H			
	mov		T6H, T6L			
	mov		T6L, T2L			
	sbc		T6L, T6H			
	clr		T6H					
	sbi		DDRC,2							; Set Vth to 0.17 Vcc.
	sbi		DDRD,6							; Discharge capacitor
	
	ldi		AL, (1<< TOIE0)					; Disable Irq. TOIE1 auf 0 aber TOIE0 1
	sts		TIMSK0,AL
	sts		TIMSK1,Zero
	sbr		Flags, bit1						; Flag: Messung-Ende

tc1c_e:
	pop		AL
	out		SREG, AL
	pop		AL
	reti


;****** Timer0 Overflow Interrupt *******************************************
;
; - TastenScan
; - Decrementieren von Stm1 und Stm2 (250Hz)


tc0_ovf:
	push	AL					
	ldi		Al,-39
	out		TCNT0, AL
	sei
	in		AL, SREG
	pushw	A
	pushw	Z

	ldi		ZL, low(DispPtr)				
	ldi		ZH, high(DispPtr)	
	ld		AH, Z							
	inc		AH								
	cpi		AH, 4				
	brcs	t0								
	rcall	scan_key			
	clr		AH
t0:	st		Z+, AH								
	popw	Z
	popw	A
	out		SREG, AL
	pop		AL
	reti

scan_key:
	in		AL, PINC						
	andi	AL, 8				;Bit3 ausmaskieren
	com		AL									
	andi	AL, bit4	      					
	ldd		AH, Z+8		   						
	std		Z+8, AL		
	cp		AL, AH		
	brne	PC+7			
	ldd		AH, Z+7		
	std		Z+7, AL
	eor		AH, AL		
	and		AH, AL		
	breq	PC+2			
	sbr		Flags, bit7			; Flag 'Button_pressed' setzen

	tst		Stm1				; 
	breq	PC+2			
	dec		Stm1		
	tst		Stm2		
	breq	PC+2		
	dec		Stm2		

	ret


;****** Spannungs-Ausgabe ***************************************************
; Ausgabe der in Bin2Ascii ausgerechneten ADC-Spannung
; Eingabe: CL, BH:BL,AH:AL / Rckgabe: keine

Print_voltage:										
 	
	push 	AL						; zuerst 1er Stelle sichern
 	push	AH                      ; 10er Stelle
 	push	BL						; 100er Stelle
	ldi		AL,0x40					; Locate 1. Stelle 2. Zeile
	rcall 	lcd_locate				; "ESR:" ausgeben
	ldi		ZL,low(form1*2)
	ldi		ZH,high(form1*2)
	rcall	lcd_print
	mov		AL,BH					; 1000er Stelle bernehmen
	rcall	lcd_data
	pop 	AL						; 100er Stelle holen
 	rcall 	lcd_data            	; ausgeben
 	ldi		Al,','               	; Komma
	rcall 	lcd_data            	; ausgeben 	
 	pop   	AL		               	; 10er Stelle holen
 	rcall 	lcd_data            	; ausgeben
 	pop   	AL					 	; 1er Stelle holen
 	rcall 	lcd_data            	; ausgeben
 	ldi   	AL,' '					; Leerstelle dranhngen
 	rcall 	lcd_data             	; ausgeben
 	ldi   	AL, 0xF4  				; Ohm dranhngen
 	rcall 	lcd_data             	; ausgeben
	ret




;****************************************************************************
; Spannungsmessung starten
; Eingabe: AL = ADMUX / Rckgabe Spannung in BL, BH, (CL)
;****************************************************************************

get_voltage:
	
	sts		ADMUX, AL			; ADC Channel setzen
	rcall	wait5ms				; warte 1ms
	clr		BL
	clr		BH					; Additionsregister lschen
	clr		CL
	
	ldi		AL, (1<< ADEN)|(1<<ADSC)|(1<<ADPS1)|(1<<ADPS0)	; Teiler 8
	sts		ADCSRA, AL

m_dummy:								
	lds		AL, ADCSRA
	sbrc	AL, ADSC
	rjmp	m_dummy
	
	clr		AH					; Schleifenzhler auf 0

m_start:
	ldi		AL, (1<< ADEN)|(1<<ADSC)|(1<<ADPS1)|(1<<ADPS0)	; Teiler 8
	sts		ADCSRA, AL 			;2. Start
	
m_wait:
	lds		AL,ADCSRA
	sbrc	AL,ADSC
	rjmp	m_wait	
	
	lds		AL, ADCL			; Messungen zwischenspeichern
	add		BL, AL				; und
	lds		AL, ADCH
	adc		BH, AL				; Werte aufaddieren
	adc		CL, Zero			; addieren ber Carry	
	dec		AH
	brne	m_start				; wiederholen bis Zhler=0

	cpi		BL, 128				; Kommastelle kleiner als 128
	brlo	m_avg				; wenn kleiner dann Sprung

	subi	BH, low(-1)			; Wert runden addieren von 1
	sbci	CL, high(-1)		; addieren des Carry

m_avg:
	mov		BL, BH
	mov		BH, CL

	sts		ADCSRA, Zero
	ret									; 		



;****************************************************************************
; Binrwert in Ascii-Code wandeln
; Eingabe: AH:AL / Rckgabe in  CL, BH:BL, AH:AL = 5 digits (ASCII)


bin2ascii:
	ldi  	CL, -1 + '0'
bcd1:
	inc  	CL
   	subi 	AL, low(10000)
   	sbci 	AH, high(10000)
   	brcc 	bcd1
   	ldi  	BH, 10 + '0'

bcd2:
	dec		BH
   	subi 	AL, low(-1000)
   	sbci 	AH, high(-1000)
   	brcs 	bcd2
   	ldi  	BL, -1 + '0'

bcd3:
	inc  	BL
   	subi 	AL, low(100)
   	sbci 	AH, high(100)
   	brcc 	bcd3
   	ldi  	AH, 10 + '0'

bcd4:
	dec  	AH
   	subi 	AL, -10
   	brcs 	bcd4
   	subi 	AL, -'0'
	ret


;****** LCD-Routinen  4-Bit *************************************************

;sendet Datenbyte an das LCD
lcd_data:
   	
	push	AL
	swap  	AL				   	   				; Obere 4 bits (Nibble) nach unten
  	andi  	AL, 0b00001111         				; 0bere Hlfte auf Null setzen
	sbr		AL, (1<<PIN_LCD_RS)					; PIN_LCD_RS						
	out		LCD_PORT,AL
	rcall 	lcd_enable             				; Enable-Routine aufrufen
   	pop		AL											; 2. Nibble	
	andi  	AL, 0b00001111         				; obere Hlfte auf Null setzen
	sbr		AL, (1<<PIN_LCD_RS)					; PIN_LCD_RS
	out		LCD_Port,AL
	rcall 	lcd_enable             				; Enable-Routine aufrufen
	rcall 	wait50us              				; warte
	ret                            				; zurck zum Hauptprogramm

;sendet einen Befehl an das LCD
lcd_command:                            		; wie lcd_data, nur ohne RS zu setzen
    		
	cbi		LCD_Port, PIN_LCD_RS
	push	AL
	swap	AL
	andi	AL, 0b00001111
	out		LCD_Port,AL
	rcall	lcd_enable
	pop		AL
	andi	AL, 0b00001111
	out		LCD_Port,AL
	rcall	lcd_enable
	rcall	wait50us
	ret


 ;erzeugt den Enable-Puls
lcd_enable:
 	sbi 	LCD_Port, PIN_LCD_Enable    		; Enable high
	nop                          				; 3 Taktzyklen warten
	nop
	nop
	cbi		LCD_Port, PIN_LCD_Enable     		; Enable wieder low
	ret                          				; und wieder zurck


 ;Initialisierung
lcd_init:
	ldi		Al,6
powerupwait:
	rcall	wait5ms
	dec		Al
	brne	powerupwait

	ldi		AL, 0b00000011         		
   	out		LCD_Port, AL              		
	rcall	lcd_enable             				; 1. muss 3mal hintereinander gesendet
   	rcall	wait5ms								;    werden zur Initialisierung
	rcall	lcd_enable             				; 2.
   	rcall	wait5ms
	rcall	lcd_enable             				; und 3!
   	rcall	wait5ms
	
	ldi		AL,	0b00000010						; 4bit Modus einstellen
	out		LCD_Port,AL
	rcall	lcd_enable
	rcall	Wait5ms	

	ldi		AL, 0b00101000         				; 4bit-Modus, 2 Zeilen
   	rcall	lcd_command							; Befehl senden
	ldi		AL, 0b00001100         				; LCD on
   	rcall	lcd_command							; Befehl senden
	ldi		AL, 0b00000100         				; Entry mode
   	rcall	lcd_command
	ret

;Display lschen
lcd_clear:
   	ldi	AL, 0b00000001         					; Display lschen
	rcall	lcd_command							; 
   	rcall	Wait5ms
	ret


;Setzt Cursor-Position auf Anfang
lcd_home:
	ldi		AL, 0b00000010         				; Cursor an Anfang
	rcall	lcd_command
  	rcall	wait5ms
	ret	

; Cursor setzen
lcd_locate:
	sbr		AL, 0b10000000         				; Bit 7 addieren
	rcall	lcd_command
	rcall	wait5ms
	ret

; Text-Ausgabe
lcd_print:
	lpm											
	tst		R0									
	breq	lcd_pend							
	mov		AL,R0								
	rcall	lcd_data							
	adiw	ZL,1              					
	rjmp	lcd_print				
lcd_pend:
	ret


;****** Warteschleifen *******************************************************

Wait5ms:;
	push	AL
	push	AH
	ldi		AL, (XTAL*5/607)/1000
n1: ldi		AH,0xC9
n2:	dec		AH
	brne	n2
    dec		AL
    brne	n1
	pop		AH
	pop		AL
	ret

Wait1s:
	push	Al
	ldi		Al,9
n6:	rcall	wait100ms
	dec		AL
	brne	n6
	pop		AL
	ret

Wait100ms:
	push	AL
	ldi		AL,19
n3:	rcall	wait5ms
	dec		AL
	brne	n3
	pop		AL
	ret


wait50us:	
	push	AL
	ldi		AL,(XTAL*50/3)/1000000
n5:	dec		AL
	brne	n5
	pop		AL
	ret


;****** Texte ***************************************************************


Buffer:	.db "           ",0

form0:	.db "Cap: ",0
form1:	.db	"ESR: ",0

form4:	.db "Kalibrierung!  ",0			

form3:	.db	"EEPROM-Fehler! ",0

form2:	.db 10,0,16,0,13,17 	;-0,0pF
		.db 0,16,0,15,13,17		;0,0 pF
		.db 0,0,16,0,13,17   	;00,0pF
		.db 15,0,0,0,13,17		; 000pF
		.db 0,16,0,0,12,17		;0,00nF
		.db 0,0,16,0,12,17		;00,0nF
		.db 15,0,0,0,12,17   	; 000nF
		.db 0,16,0,0,11,17		;0,00uF
		.db 0,0,16,0,11,17		;00,0uF
		.db 15,0,0,0,11,17		; 000uF
		.db 0,0,0,0,11,17		;0000uF
		.db 15,19,18,15,15,15 	; 0L
		.db 15,19,18,15,15,15 	; 0L
		.db 15,19,18,15,15,15 	; 0L

seg7:	.db '0','1','2','3','4','5','6','7','8','9'
		.db '-',0xE4,'n',0xE6,'E',' ',',','F','L','O'
		;    10,  11, 12, 13, 14, 15, 16, 17, 18, 19
		
Overflow:
		.db "OL     ",0
Leer:	
		.db "             ",0

